﻿using System;
using System.Runtime.InteropServices;

namespace SDRSharp.FrontEnds.SpyServer
{
    public static class Constants
    {
        public const uint SPYSERVER_PROTOCOL_VERSION = (((2) << 24) | ((0) << 16) | (1570));
        public const uint SPYSERVER_MAX_COMMAND_BODY_SIZE = (256);     // 256 B
        public const uint SPYSERVER_MAX_MESSAGE_BODY_SIZE = (1 << 20); //  1 MB
        public const uint SPYSERVER_MAX_DISPLAY_PIXELS = (1 << 15);    // 32k
        public const uint SPYSERVER_MIN_DISPLAY_PIXELS = (100);        // 32k
        public const uint SPYSERVER_MAX_FFT_DB_RANGE = (150);          // 150 dB
        public const uint SPYSERVER_MIN_FFT_DB_RANGE = (10);           // 10 dB
        public const uint SPYSERVER_MAX_FFT_DB_OFFSET = (100);         // 100 dB

        public const uint SPYSERVER_MESSAGE_TYPE_MASK = 0x0000ffff;
        public const uint SPYSERVER_MESSAGE_FLAG_MASK = 0xffff0000;

        public static string GetDeviceName(DeviceType deviceID)
        {
            switch (deviceID)
            {
                case DeviceType.DEVICE_AIRSPY_ONE:
                    return "Airspy One";

                case DeviceType.DEVICE_AIRSPY_HF:
                    return "Airspy HF+";

                case DeviceType.DEVICE_RTLSDR:
                    return "RTL-SDR";

                default:
                    return "Unknown";
            }
        }
    }

    public enum DeviceType : uint
    {
	    DEVICE_INVALID				= 0,
	    DEVICE_AIRSPY_ONE			= 1,
	    DEVICE_AIRSPY_HF			= 2,
	    DEVICE_RTLSDR				= 3,
    }

    public enum CommandType : uint
    {
	    CMD_HELLO					= 0,
	    CMD_GET_SETTING				= 1,
	    CMD_SET_SETTING				= 2,
	    CMD_PING					= 3,
    }

    public enum SettingType : uint
    {
	    SETTING_STREAMING_MODE		= 0,
	    SETTING_STREAMING_ENABLED	= 1,
	    SETTING_GAIN				= 2,

        SETTING_IQ_FORMAT			= 100,
	    SETTING_IQ_FREQUENCY		= 101,
	    SETTING_IQ_DECIMATION		= 102,
        SETTING_IQ_DIGITAL_GAIN     = 103,

        SETTING_FFT_FORMAT			= 200,
	    SETTING_FFT_FREQUENCY		= 201,
	    SETTING_FFT_DECIMATION		= 202,
	    SETTING_FFT_DB_OFFSET		= 203,
	    SETTING_FFT_DB_RANGE		= 204,
	    SETTING_FFT_DISPLAY_PIXELS	= 205,
    }

    public enum StreamType : uint
    {
	    STREAM_TYPE_STATUS			= 0,
	    STREAM_TYPE_IQ				= 1,
	    STREAM_TYPE_AF				= 2,
	    STREAM_TYPE_FFT				= 4,
    }

    public enum StreamingMode : uint
    {
	    STREAM_MODE_IQ_ONLY			= StreamType.STREAM_TYPE_IQ,
        STREAM_MODE_AF_ONLY         = StreamType.STREAM_TYPE_AF,
        STREAM_MODE_FFT_ONLY        = StreamType.STREAM_TYPE_FFT,
        STREAM_MODE_FFT_IQ          = StreamType.STREAM_TYPE_FFT | StreamType.STREAM_TYPE_IQ,
        STREAM_MODE_FFT_AF          = StreamType.STREAM_TYPE_FFT | StreamType.STREAM_TYPE_AF,
    }

    public enum StreamFormat : uint
    {
	    STREAM_FORMAT_DINT4			= 0,
	    STREAM_FORMAT_UINT8			= 1,
	    STREAM_FORMAT_INT16			= 2,
	    STREAM_FORMAT_INT24			= 3,
	    STREAM_FORMAT_FLOAT			= 4,

        STREAM_FORMAT_COMPRESSED    = 1 << 16,
    }

    [Flags]
    public enum MessageType : uint
    {
	    MSG_TYPE_DEVICE_INFO		= 0,
	    MSG_TYPE_CLIENT_SYNC		= 1,
	    MSG_TYPE_PONG				= 2,
	    MSG_TYPE_READ_SETTING		= 3,

	    MSG_TYPE_UINT8_IQ			= 100,
	    MSG_TYPE_INT16_IQ			= 101,
	    MSG_TYPE_INT24_IQ			= 102,
	    MSG_TYPE_FLOAT_IQ			= 103,

	    MSG_TYPE_UINT8_AF			= 200,
	    MSG_TYPE_INT16_AF			= 201,
	    MSG_TYPE_INT24_AF			= 202,
	    MSG_TYPE_FLOAT_AF			= 203,

	    MSG_TYPE_DINT4_FFT			= 300,
	    MSG_TYPE_UINT8_FFT			= 301,

        MSG_TYPE_COMPRESSED         = 1 << 16,
    }

    enum CompressionType : uint
    {
        COMPRESSION_TYPE_NONE       = 0,
        COMPRESSION_TYPE_ARITHMETIC = 1,
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct ClientHandshake
    {
        public uint ProtocolVersion;
        public uint ClientNameLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct CommandHeader
    {
        public CommandType CommandType;
        public uint BodySize;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct SettingTarget
    {
        public StreamType StreamType;
        public SettingType SettingType;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MessageHeader
    {
        public uint ProtocolID;
        public MessageType MessageType;
        public StreamType StreamType;
        public uint SequenceNumber;
        public uint BodySize;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct DeviceInfo
    {
        public DeviceType DeviceType;
        public uint DeviceSerial;
        public uint MaximumSampleRate;
        public uint MaximumBandwidth;
        public uint DecimationStageCount;
        public uint GainStageCount;
        public uint MaximumGainIndex;
        public uint MinimumFrequency;
        public uint MaximumFrequency;
        public uint Resolution;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct ClientSync
    {
        public uint CanControl;
        public uint Gain;
        public uint DeviceCenterFrequency;
        public uint IQCenterFrequency;
        public uint FFTCenterFrequency;
        public uint MinimumIQCenterFrequency;
        public uint MaximumIQCenterFrequency;
        public uint MinimumFFTCenterFrequency;
        public uint MaximumFFTCenterFrequency;
    }

    [StructLayout(LayoutKind.Sequential)]
    struct CompressionHeader
    {
        public CompressionType CompressionType;
        public uint UncompressedSize;
    }
}
